--[[ Credit: Blazeflack]]

--Add access to ElvUI engine and unitframe framework
local E = unpack(ElvUI);
local ElvUF = ElvUI.oUF
assert(ElvUF, "ElvUI was unable to locate oUF.")

--Cache global variables
--Lua functions
local _G = _G
local unpack, pairs, assert = unpack, pairs, assert
local twipe = table.wipe
local ceil, sqrt, floor, abs = math.ceil, math.sqrt, math.floor, math.abs
local format, strupper, len = string.format, strupper, string.len

--WoW API
local GetNumGroupMembers = GetNumGroupMembers
local IsInGroup = IsInGroup
local IsInRaid = IsInRaid
local UnitClass = UnitClass
local UnitFactionGroup = UnitFactionGroup
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitIsDead = UnitIsDead
local UnitIsGhost = UnitIsGhost
local UnitIsUnit = UnitIsUnit
local UnitName = UnitName
local UnitPower = UnitPower
local UnitPowerMax = UnitPowerMax
local UnitPowerType = UnitPowerType
local UnitIsPlayer = UnitIsPlayer
local SPELL_POWER_MANA = Enum.PowerType.Mana
local GetSpecialization = GetSpecialization
local GetSpecializationInfo = GetSpecializationInfo

--GLOBALS: _TAGS, Hex, _COLORS

local textFormatStyles = {
	["CURRENT"] = "%s",
	["CURRENT_MAX"] = "%s - %s",
	["CURRENT_PERCENT"] =  "%s - %.1f%%",
	["CURRENT_MAX_PERCENT"] = "%s - %s | %.1f%%",
	["PERCENT"] = "%.1f%%",
	["DEFICIT"] = "-%s"
}

local textFormatStylesNoDecimal = {
	["CURRENT"] = "%s",
	["CURRENT_MAX"] = "%s - %s",
	["CURRENT_PERCENT"] =  "%s - %.0f%%",
	["CURRENT_MAX_PERCENT"] = "%s - %s | %.0f%%",
	["PERCENT"] = "%.0f%%",
	["DEFICIT"] = "-%s"
}

local shortValueFormat
local function ShortValue(number, noDecimal)
	shortValueFormat = (noDecimal and "%.0f%s" or format("%%.%df", E.db.general.decimalLength or 1))
	if E.db.general.numberPrefixStyle == "METRIC" then
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "T")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "G")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "M")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "k")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "CHINESE" then
		if abs(number) >= 1e8 then
			return format("%.1f%s", number / 1e8, "Y")
		elseif abs(number) >= 1e4 then
			return format("%.1f%s", number / 1e4, "W")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "KOREAN" then
		if abs(number) >= 1e8 then
			return format("%.1f%s", number / 1e8, "억")
		elseif abs(number) >= 1e4 then
			return format("%.1f%s", number / 1e4, "만")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "천")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "GERMAN" then
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "Bio")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "Mrd")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "Mio")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "Tsd")
		else
			return format("%.0f", number)
		end
	else
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "T")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "B")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "M")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "K")
		else
			return format("%.0f", number)
		end
	end
end

local function GetFormattedText(min, max, style, noDecimal)
	assert(textFormatStyles[style] or textFormatStylesNoDecimal[style], "CustomTags Invalid format style: "..style)
	assert(min, "CustomTags - You need to provide a current value. Usage: GetFormattedText(min, max, style, noDecimal)")
	assert(max, "CustomTags - You need to provide a maximum value. Usage: GetFormattedText(min, max, style, noDecimal)")

	if max == 0 then max = 1 end

	local chosenFormat
	if noDecimal then
		chosenFormat = textFormatStylesNoDecimal[style]
	else
		chosenFormat = textFormatStyles[style]
	end

	if style == "DEFICIT" then
		local deficit = max - min
		if deficit <= 0 then
			return nil
		else
			return format(chosenFormat, ShortValue(deficit, noDecimal))
		end
	elseif style == "PERCENT" then
		return format(chosenFormat, min / max * 100)
	elseif style == "CURRENT" or ((style == "CURRENT_MAX" or style == "CURRENT_MAX_PERCENT" or style == "CURRENT_PERCENT") and min == max) then
		if noDecimal then
			return format(textFormatStylesNoDecimal["CURRENT"], ShortValue(min, noDecimal))
		else
			return format(textFormatStyles["CURRENT"], ShortValue(min, noDecimal))
		end
	elseif style == "CURRENT_MAX" then
		return format(chosenFormat, ShortValue(min, noDecimal), ShortValue(max, noDecimal))
	elseif style == "CURRENT_PERCENT" then
		return format(chosenFormat, ShortValue(min, noDecimal), min / max * 100)
	elseif style == "CURRENT_MAX_PERCENT" then
		return format(chosenFormat, ShortValue(min, noDecimal), ShortValue(max, noDecimal), min / max * 100)
	end
end

--[[
	Custom Tags
--]]

E:AddTagInfo("gnuihmana", "|cffc41f3bGennUI|r", "Mana percent for Healers only, hide when full", 3)
ElvUF.Tags.Events['gnuihmana'] = 'UNIT_POWER_FREQUENT UNIT_MAXPOWER'
ElvUF.Tags.Methods['gnuihmana'] = function(unit)
	local min = UnitPower(unit, SPELL_POWER_MANA)
	local max = UnitPowerMax(unit, SPELL_POWER_MANA)
	local currentSpec = GetSpecialization()
	local role = UnitGroupRolesAssigned(unit)
	if currentSpec then
		local _, currentSpecName = GetSpecializationInfo(currentSpec)
		if (currentSpecName == "Restoration" or  currentSpecName == "Mistweaver" or currentSpecName == "Discipline" or currentSpecName == "Holy") and min ~= max then
		return E:GetFormattedText('PERCENT', UnitPower(unit, SPELL_POWER_MANA), UnitPowerMax(unit, SPELL_POWER_MANA))
		end
	end
	if role == 'HEALER' and min ~= max then
		return E:GetFormattedText('PERCENT', UnitPower(unit, SPELL_POWER_MANA), UnitPowerMax(unit, SPELL_POWER_MANA))
	end
end

E:AddTagInfo("gnuipower", "|cffc41f3bGennUI|r", "Power short value, hide when zero", 4)
ElvUF.Tags.Events['gnuipower'] = 'UNIT_DISPLAYPOWER UNIT_POWER_FREQUENT UNIT_MAXPOWER'
ElvUF.Tags.Methods['gnuipower'] = function(unit)
	local pType = UnitPowerType(unit)
	local min = UnitPower(unit, pType)
	local max = UnitPowerMax(unit, pType)
	if min ~= 0 then
	return E:GetFormattedText('CURRENT', UnitPower(unit, pType), UnitPowerMax(unit, pType), nil, true)
	end
end